define(["exports", "core/post-process", "core/pre-process", "core/pubsubhub", "core/include-config", "core/override-configuration", "core/remove-respec", "core/respec-ready"], function (exports, _postProcess, _preProcess, _pubsubhub) {
  "use strict";

  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  exports.runAll = exports.name = undefined;

  function _asyncToGenerator(fn) {
    return function () {
      var gen = fn.apply(this, arguments);
      return new Promise(function (resolve, reject) {
        function step(key, arg) {
          try {
            var info = gen[key](arg);
            var value = info.value;
          } catch (error) {
            reject(error);
            return;
          }

          if (info.done) {
            resolve(value);
          } else {
            return Promise.resolve(value).then(function (value) {
              step("next", value);
            }, function (err) {
              step("throw", err);
            });
          }
        }

        return step("next");
      });
    };
  }

  const name = exports.name = "core/base-runner";
  const canMeasure = performance.mark && performance.measure;

  function toRunnable(plug) {
    const name = plug.name || "";
    if (!name) {
      console.warn("Plugin lacks name:", plug);
    }
    return config => {
      return new Promise((() => {
        var _ref = _asyncToGenerator(function* (resolve, reject) {
          const timerId = setTimeout(function () {
            const msg = `Plugin ${name} took too long.`;
            console.error(msg, plug);
            reject(new Error(msg));
          }, 5000);
          // Modern plugins are async or normal functions, take one argument (conf)
          if (canMeasure) {
            performance.mark(name + "-start");
          }
          try {
            if (plug.run.length === 1) {
              yield plug.run(config);
              resolve();
            } else {
              plug.run(config, document, resolve);
            }
          } catch (err) {
            reject(err);
          } finally {
            clearTimeout(timerId);
          }
          if (canMeasure) {
            performance.mark(name + "-end");
            performance.measure(name, name + "-start", name + "-end");
          }
        });

        return function (_x, _x2) {
          return _ref.apply(this, arguments);
        };
      })());
    };
  }

  let runAll = exports.runAll = (() => {
    var _ref2 = _asyncToGenerator(function* (plugs) {
      (0, _pubsubhub.pub)("start-all", respecConfig);
      if (canMeasure) {
        performance.mark(name + "-start");
      }
      yield _preProcess.done;
      const runnables = plugs.filter(function (plug) {
        return plug.run;
      }).map(toRunnable);
      for (const task of runnables) {
        try {
          yield task(respecConfig);
        } catch (err) {
          console.error(err);
        }
      }
      (0, _pubsubhub.pub)("plugins-done", respecConfig);
      yield _postProcess.done;
      (0, _pubsubhub.pub)("end-all", respecConfig);
      if (canMeasure) {
        performance.mark(name + "-end");
        performance.measure(name, name + "-start", name + "-end");
      }
    });

    return function runAll(_x3) {
      return _ref2.apply(this, arguments);
    };
  })();
});
//# sourceMappingURL=base-runner.js.map