define(["exports", "core/github", "core/pubsubhub"], function (exports, _github, _pubsubhub) {
  "use strict";

  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  exports.run = exports.name = undefined;

  function _asyncToGenerator(fn) {
    return function () {
      var gen = fn.apply(this, arguments);
      return new Promise(function (resolve, reject) {
        function step(key, arg) {
          try {
            var info = gen[key](arg);
            var value = info.value;
          } catch (error) {
            reject(error);
            return;
          }

          if (info.done) {
            resolve(value);
          } else {
            return Promise.resolve(value).then(function (value) {
              step("next", value);
            }, function (err) {
              step("throw", err);
            });
          }
        }

        return step("next");
      });
    };
  }

  const name = exports.name = "core/contrib";

  function prop(prop) {
    return function (o) {
      return o[prop];
    };
  }

  function findUsers(...thingsToFind) {
    var users = {};
    thingsToFind.forEach(function (things) {
      things.forEach(function (thing) {
        if (thing.user) {
          users[thing.user.url] = true;
        }
      });
    });
    return Object.keys(users);
  }

  function join(things) {
    if (!things.length) {
      return "";
    }
    things = things.slice(0);
    var last = things.pop();
    var length = things.length;
    if (length === 0) {
      return last;
    }
    if (length === 1) {
      return things[0] + " and " + last;
    }
    return things.join(", ") + ", and " + last;
  }

  function toHTML(urls, editors, element) {
    return $.when.apply($, urls.map(function (url) {
      return (0, _github.fetch)(url);
    })).then(function (...args) {
      var names = args.map(function (user) {
        user = user[0];
        return user.name || user.login;
      }).filter(function (name) {
        return editors.indexOf(name) < 0;
      });
      names.sort(function (a, b) {
        return a.toLowerCase().localeCompare(b.toLowerCase());
      });
      $(element).html(join(names)).attr("id", null);
    });
  }

  let run = exports.run = (() => {
    var _ref = _asyncToGenerator(function* (conf, doc, cb) {
      var $commenters = doc.querySelector("#gh-commenters");
      var $contributors = doc.querySelector("#gh-contributors");

      if (!$commenters && !$contributors) {
        return cb();
      }

      if (!conf.githubAPI) {
        var elements = [];
        if ($commenters) elements.push("#" + $commenters.id);
        if ($contributors) elements.push("#" + $contributors.id);
        (0, _pubsubhub.pub)("error", "Requested list of contributors and/or commenters from GitHub (" + elements.join(" and ") + ") but config.githubAPI is not set.");
        cb();
        return;
      }

      (0, _github.fetch)(conf.githubAPI).then(function (json) {
        return $.when((0, _github.fetchIndex)(json.issues_url), (0, _github.fetchIndex)(json.issue_comment_url), (0, _github.fetchIndex)(json.contributors_url));
      }).then(function (issues, comments, contributors) {
        var editors = respecConfig.editors.map(prop("name"));
        var commenters = findUsers(issues, comments);
        contributors = contributors.map(prop("url"));
        return $.when(toHTML(commenters, editors, $commenters), toHTML(contributors, editors, $contributors));
      }).then(cb, function (error) {
        (0, _pubsubhub.pub)("error", "Error loading contributors and/or commenters from  Error: " + error);
        cb();
      });
    });

    return function run(_x, _x2, _x3) {
      return _ref.apply(this, arguments);
    };
  })();
});
//# sourceMappingURL=contrib.js.map