define(["exports", "core/pubsubhub", "core/biblio"], function (exports, _pubsubhub, _biblio) {
  "use strict";

  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  exports.linkInlineCitations = exports.run = exports.name = undefined;

  function _asyncToGenerator(fn) {
    return function () {
      var gen = fn.apply(this, arguments);
      return new Promise(function (resolve, reject) {
        function step(key, arg) {
          try {
            var info = gen[key](arg);
            var value = info.value;
          } catch (error) {
            reject(error);
            return;
          }

          if (info.done) {
            resolve(value);
          } else {
            return Promise.resolve(value).then(function (value) {
              step("next", value);
            }, function (err) {
              step("throw", err);
            });
          }
        }

        return step("next");
      });
    };
  }

  const name = exports.name = "core/data-cite";

  function requestLookup(conf) {
    const toCiteDetails = citeDetailsConverter(conf);
    return (() => {
      var _ref = _asyncToGenerator(function* (elem) {
        const originalKey = elem.dataset.cite;
        let { key, frag, path } = toCiteDetails(elem);
        let href = "";
        // This is just referring to this document
        if (key === conf.shortName) {
          href = document.location.href;
        } else {
          // Let's go look it up in spec ref...
          const entry = yield (0, _biblio.resolveRef)(key);
          cleanElement(elem);
          if (!entry) {
            var msg = `Couldn't find a match for 'data-cite=${originalKey}'.`;
            console.warn(msg, elem);
            msg += " Please check developer console for offending element.";
            (0, _pubsubhub.pub)("warn", msg);
            return;
          }
          href = entry.href;
        }
        if (path) {
          href = new URL(path, href).href;
        }
        if (frag) {
          href = new URL(frag, href).href;
        }
        switch (elem.localName) {
          case "a":
            {
              elem.href = href;
              break;
            }
          case "dfn":
            {
              const a = elem.ownerDocument.createElement("a");
              a.href = href;
              while (elem.firstChild) {
                a.appendChild(elem.firstChild);
              }
              elem.appendChild(a, elem);
              break;
            }
        }
      });

      return function (_x) {
        return _ref.apply(this, arguments);
      };
    })();
  }

  function cleanElement(elem) {
    ["data-cite", "data-cite-frag"].filter(attrName => elem.hasAttribute(attrName)).forEach(attrName => elem.removeAttribute(attrName));
  }

  function citeDetailsConverter(conf) {
    return function toCiteDetails(elem) {
      const { dataset } = elem;
      let { cite: key, citeFrag: frag, citePath: path } = dataset;
      const isNormative = key.startsWith("!");
      const pathPosition = key.search("/");
      const fragPosition = key.search("#");
      // The key is a fragment, resolve using the shortName as key
      if (key.startsWith("#") && !frag) {
        // Closes data-cite not starting with "#"
        const closest = elem.parentElement.closest(`[data-cite]:not([data-cite^="#"])`);
        const { key: parentKey, isNormative: closestIsNormative } = closest ? toCiteDetails(closest) : { key: conf.shortName || "", isNormative: false };
        elem.dataset.cite = closestIsNormative ? `!${parentKey}` : parentKey;
        elem.dataset.citeFrag = key; // the key is acting as fragment
        return toCiteDetails(elem);
      }
      if (fragPosition !== -1) {
        frag = !frag ? key.substr(fragPosition) : frag;
        key = key.substring(0, fragPosition);
      }
      if (pathPosition !== -1) {
        path = !path ? key.substr(pathPosition) : path;
        key = key.substring(0, pathPosition);
      }
      if (isNormative) {
        key = key.substr(1);
      }
      if (frag && !frag.startsWith("#")) {
        frag = "#" + frag;
      }
      // remove head / for URL resolution
      if (path && path.startsWith("/")) {
        path = path.substr(1);
      }
      return { key, isNormative, frag, path };
    };
  }

  let run = exports.run = (() => {
    var _ref2 = _asyncToGenerator(function* (conf) {
      const toCiteDetails = citeDetailsConverter(conf);
      Array.from(document.querySelectorAll(["dfn[data-cite], a[data-cite]"])).filter(function (el) {
        return el.dataset.cite;
      }).map(toCiteDetails).forEach(function ({ isNormative, key }) {
        const refSink = isNormative ? conf.normativeReferences : conf.informativeReferences;
        refSink.add(key);
      });
    });

    return function run(_x2) {
      return _ref2.apply(this, arguments);
    };
  })();

  let linkInlineCitations = exports.linkInlineCitations = (() => {
    var _ref3 = _asyncToGenerator(function* (doc, conf = respecConfig) {
      const toLookupRequest = requestLookup(conf);
      const citedSpecs = doc.querySelectorAll("dfn[data-cite], a[data-cite]");
      const lookupRequests = Array.from(citedSpecs).map(toLookupRequest);
      return yield Promise.all(lookupRequests);
    });

    return function linkInlineCitations(_x3) {
      return _ref3.apply(this, arguments);
    };
  })();
});
//# sourceMappingURL=data-cite.js.map