define(["exports", "deps/text!core/css/github.css", "core/pubsubhub", "core/worker"], function (exports, _github, _pubsubhub, _worker) {
  "use strict";

  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  exports.run = exports.name = undefined;

  var _github2 = _interopRequireDefault(_github);

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  function _asyncToGenerator(fn) {
    return function () {
      var gen = fn.apply(this, arguments);
      return new Promise(function (resolve, reject) {
        function step(key, arg) {
          try {
            var info = gen[key](arg);
            var value = info.value;
          } catch (error) {
            reject(error);
            return;
          }

          if (info.done) {
            resolve(value);
          } else {
            return Promise.resolve(value).then(function (value) {
              step("next", value);
            }, function (err) {
              step("throw", err);
            });
          }
        }

        return step("next");
      });
    };
  }

  const name = exports.name = "core/highlight";

  // Opportunistically insert the style into the head to reduce FOUC.
  const codeStyle = document.createElement("style");
  codeStyle.textContent = _github2.default;
  document.head.appendChild(codeStyle);
  let idCounter = 0;
  function getLanguageHint(classList) {
    return Array.from(classList).filter(item => item !== "highlight" && item !== "nolinks").map(item => item.toLowerCase());
  }

  let run = exports.run = (() => {
    var _ref = _asyncToGenerator(function* (conf) {
      // Nothing to do
      if (conf.noHighlightCSS) {
        return;
      }
      const highlightables = Array.from(document.querySelectorAll("pre:not(.idl):not(.nohighlight), code.highlight"));
      if (!highlightables.length) {
        return;
      }
      const promisesToHighlight = highlightables.map(function (element) {
        return new Promise(function (resolve) {
          if (element.textContent.trim() === "") {
            return resolve(element); // no work to do
          }
          const done = function () {
            element.setAttribute("aria-busy", "false");
            resolve(element);
          };
          // We always resolve, even if we couldn't actually highlight
          const timeoutId = setTimeout(function () {
            console.error("Timed-out waiting for highlight:", element);
            done();
          }, 4000);
          const msg = {
            action: "highlight",
            code: element.textContent,
            id: "highlight:" + idCounter++,
            languages: getLanguageHint(element.classList)
          };
          element.setAttribute("aria-busy", "true");
          element.setAttribute("aria-live", "polite");
          _worker.worker.postMessage(msg);
          _worker.worker.addEventListener("message", function listener(ev) {
            const { data: { id, code, language, value } } = ev;
            if (id !== msg.id) {
              return; // not for us!
            }
            element.innerHTML = value;
            if (element.localName === "pre") {
              element.classList.add("hljs");
            }
            if (language) {
              element.classList.add(language);
            }
            clearTimeout(timeoutId);
            _worker.worker.removeEventListener("message", listener);
            done();
          });
        });
      });
      yield Promise.all(promisesToHighlight);
    });

    return function run(_x) {
      return _ref.apply(this, arguments);
    };
  })();
});
//# sourceMappingURL=highlight.js.map