define(["exports", "core/pubsubhub"], function (exports, _pubsubhub) {
  "use strict";

  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  exports.run = exports.name = undefined;

  var _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  function _asyncToGenerator(fn) {
    return function () {
      var gen = fn.apply(this, arguments);
      return new Promise(function (resolve, reject) {
        function step(key, arg) {
          try {
            var info = gen[key](arg);
            var value = info.value;
          } catch (error) {
            reject(error);
            return;
          }

          if (info.done) {
            resolve(value);
          } else {
            return Promise.resolve(value).then(function (value) {
              step("next", value);
            }, function (err) {
              step("throw", err);
            });
          }
        }

        return step("next");
      });
    };
  }

  const name = exports.name = "core/linter";
  const privates = new WeakMap();

  class Linter {
    constructor() {
      privates.set(this, {
        rules: new Set()
      });
    }
    get rules() {
      return privates.get(this).rules;
    }
    register(...newRules) {
      newRules.reduce((rules, newRule) => rules.add(newRule), this.rules);
    }
    lint(conf, doc = window.document) {
      var _this = this;

      return _asyncToGenerator(function* () {
        const promisesToLint = [...privates.get(_this).rules].map(function (rule) {
          return toLinterWarning(rule.lint(conf, doc));
        });
        yield promisesToLint;
      })();
    }
  }

  const linter = new Linter();
  exports.default = linter;


  const baseResult = {
    name: "unknown",
    description: "",
    occurrences: 0,
    howToFix: "",
    offendingElements: [], // DOM Nodes
    help: "" // where to get help
  };

  let toLinterWarning = (() => {
    var _ref = _asyncToGenerator(function* (promiseToLint) {
      const results = yield promiseToLint;
      results.map((() => {
        var _ref2 = _asyncToGenerator(function* (resultPromise) {
          const result = yield resultPromise;
          const output = _extends({}, baseResult, result);
          const {
            description,
            help,
            howToFix,
            name,
            occurrences,
            offendingElements
          } = output;
          const msg = `${description} ${howToFix} ${help} ("${name}" x ${occurrences})`;
          offendingElements.forEach(function (elem) {
            elem.classList.add("respec-offending-element");
          });
          console.warn(`Linter (${name}):`, description, ...offendingElements);
          return msg;
        });

        return function (_x2) {
          return _ref2.apply(this, arguments);
        };
      })()).forEach((() => {
        var _ref3 = _asyncToGenerator(function* (msgPromise) {
          (0, _pubsubhub.pub)("warn", (yield msgPromise));
        });

        return function (_x3) {
          return _ref3.apply(this, arguments);
        };
      })());
    });

    return function toLinterWarning(_x) {
      return _ref.apply(this, arguments);
    };
  })();

  let run = exports.run = (() => {
    var _ref4 = _asyncToGenerator(function* (conf, doc, cb) {
      cb(); // return early, continue processing other things
      if (conf.lint === false) {
        return; // nothing to do
      }
      yield document.respecReady;
      try {
        yield linter.lint(conf, doc);
      } catch (err) {
        console.error("Error ocurred while running the linter", err);
      }
    });

    return function run(_x4, _x5, _x6) {
      return _ref4.apply(this, arguments);
    };
  })();
});
//# sourceMappingURL=linter.js.map