define(["exports"], function (exports) {
  "use strict";

  Object.defineProperty(exports, "__esModule", {
    value: true
  });

  function _asyncToGenerator(fn) {
    return function () {
      var gen = fn.apply(this, arguments);
      return new Promise(function (resolve, reject) {
        function step(key, arg) {
          try {
            var info = gen[key](arg);
            var value = info.value;
          } catch (error) {
            reject(error);
            return;
          }

          if (info.done) {
            resolve(value);
          } else {
            return Promise.resolve(value).then(function (value) {
              step("next", value);
            }, function (err) {
              step("throw", err);
            });
          }
        }

        return step("next");
      });
    };
  }

  /**
   * This Module adds a metatag description to the document, based on the
   * first paragraph of the abstract.
   */

  const name = exports.name = "core/seo";

  let run = exports.run = (() => {
    var _ref = _asyncToGenerator(function* (conf, doc, cb) {
      // This is not critical, so let's continue other processing first
      cb();
      yield doc.respecIsReady;
      const firstParagraph = doc.querySelector("#abstract p:first-of-type");
      if (!firstParagraph) {
        return; // no abstract, so nothing to do
      }
      const insertMetaDescription = makeDescriptionInserter(firstParagraph);
      if (window.requestIdleCallback) {
        window.requestIdleCallback(insertMetaDescription);
      } else {
        insertMetaDescription();
      }
    });

    return function run(_x, _x2, _x3) {
      return _ref.apply(this, arguments);
    };
  })();

  function makeDescriptionInserter(firstParagraph) {
    return () => {
      // Normalize whitespace: trim, remove new lines, tabs, etc.
      const doc = firstParagraph.ownerDocument;
      const content = firstParagraph.textContent.replace(/\s+/, " ").trim();
      const metaElem = doc.createElement("meta");
      metaElem.name = "description";
      metaElem.content = content;
      doc.head.appendChild(metaElem);
    };
  }
});
//# sourceMappingURL=seo.js.map